"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SContractReader = void 0;

var _classPrivateFieldLooseBase2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseBase"));

var _classPrivateFieldLooseKey2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseKey"));

var _utilCrypto = require("@polkadot/util-crypto");

var _file2 = require("@skyekiwi/file");

var _util = require("@skyekiwi/util");

// Copyright 2021 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
var _file = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("file");

var _sealer = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("sealer");

var _contract = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("contract");

var _seed = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("seed");

class SContractReader {
  constructor(file, sealer) {
    Object.defineProperty(this, _file, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _sealer, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _contract, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _seed, {
      writable: true,
      value: void 0
    });
    (0, _classPrivateFieldLooseBase2.default)(this, _file)[_file] = file;
    (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer] = sealer;
  }

  async init() {
    try {
      const fullContract = await (0, _classPrivateFieldLooseBase2.default)(this, _file)[_file].readAll();
      (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract] = this.decodeContract((0, _util.fromBase64)(fullContract));
    } catch (err) {
      throw new Error('initialization error, file might not be found s-contract/SContractReader/init');
    }
  }

  unlockSealer(seed) {
    const key = (0, _utilCrypto.mnemonicToMiniSecret)(seed);

    (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].unlock(key);
  }

  getContractPublicKey() {
    return (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].getAuthorKey();
  }

  decodeCall(call) {
    const _call = call.split(':');

    let callContent = '';
    const callIndex = _call[0];
    let encrypted = false;

    if ((0, _util.isValidHex)(_call[1])) {
      encrypted = true;
      callContent = (0, _util.u8aToString)((0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].decrypt((0, _util.hexToU8a)(_call[1])));
    } else {
      callContent = _call[1];
    }

    const oneCall = callContent.split('?');
    return {
      callIndex: callIndex,
      contractId: oneCall[0],
      encrypted: encrypted,
      methodName: (0, _util.fromBase64)(oneCall[1]),
      origin: (0, _util.fromBase64)(oneCall[2]),
      parameters: (0, _util.fromBase64)(oneCall[3])
    };
  }

  encodeCall(call) {
    return SContractReader.rawEncodeCall(call, (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer], (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].getAuthorKey());
  }

  static rawEncodeCall(call, sealer, dest) {
    if (call.methodName.length < 0 || call.methodName.length > 32) {
      throw new Error('methodName must be between 0 - 32 bytes - calls/encodeCall');
    }

    if (call.parameters.length < 0 || call.parameters.length > 128) {
      throw new Error('parameters must be between 0 - 128 bytes - calls/encodeCall');
    }

    if (!(0, _util.isValidSubstrateAddress)(call.origin)) {
      throw new Error('origin must be a valid Substrate address - calls/encodeCall');
    }

    let callString = `${call.contractId}?${(0, _util.toBase64)(call.methodName)}?${(0, _util.toBase64)(call.origin)}?${(0, _util.toBase64)(call.parameters)}`;

    if (call.encrypted) {
      if (!dest || dest.length !== 32) {
        throw new Error('destination key missing or length error');
      }

      if (!sealer) {
        throw new Error('sealer is required for encrypted calls');
      }

      callString = (0, _util.u8aToHex)(sealer.encrypt((0, _util.stringToU8a)(callString), dest));
    }

    return call.callIndex + ':' + callString;
  }

  encodeAuth(auth) {
    return (0, _util.toBase64)(`${auth.storageKey}?${auth.authOrigin}`);
  }

  decodeAuth(authStringRaw) {
    const authString = (0, _util.fromBase64)(authStringRaw);
    const a = authString.split('?');
    const storageKey = a[0];
    const auth = a[1];
    if (storageKey === undefined) return null;
    return {
      authOrigin: auth,
      storageKey: storageKey
    };
  }

  encodeContract(seed, contract) {
    const authString = contract.auth.map(this.encodeAuth).join('|');
    const result = `${contract.contractId}*${seed}*${authString}*${contract.highLocalCallIndex}*${contract.state}*${contract.wasmPath}`;
    return result;
  }

  decodeContract(contractRaw) {
    const contract = contractRaw.split('*');
    const contractId = contract[0];
    const seed = contract[1];
    const auths = contract[2].split('|').map(this.decodeAuth);
    const highLocalCallIndex = contract[3];
    const state = contract[4];
    const wasmPath = contract[5];
    this.unlockSealer(seed);
    (0, _classPrivateFieldLooseBase2.default)(this, _seed)[_seed] = seed;
    return {
      auth: auths,
      contractId: contractId,
      highLocalCallIndex: highLocalCallIndex,
      state: state,
      wasmPath: wasmPath
    };
  }

  writeState(state) {
    (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].state = state;
  }

  readState() {
    return (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].state;
  }

  writeAuth(auth) {
    (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].auth.push(auth);
  }

  revokeAuth(auth) {
    (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].auth.forEach((v, i) => {
      if (v === auth) {
        (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].auth.splice(i, 1);
      }
    });
  }

  getContractId() {
    return (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].contractId;
  }

  async writeToFile(path) {
    await _file2.File.writeFile((0, _util.stringToU8a)((0, _util.toBase64)(this.encodeContract((0, _classPrivateFieldLooseBase2.default)(this, _seed)[_seed], (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract]))), path, 'w');
  }

  forceInject(seed, contract) {
    (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract] = contract;
    (0, _classPrivateFieldLooseBase2.default)(this, _seed)[_seed] = seed;

    (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].unlock((0, _utilCrypto.mnemonicToMiniSecret)(seed));
  }

  getHighLocalCallIndex() {
    return (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract].highLocalCallIndex;
  }

  readContract() {
    return (0, _classPrivateFieldLooseBase2.default)(this, _contract)[_contract];
  }

}

exports.SContractReader = SContractReader;